@@ -7,7 +7,9 @@
 #include <unistd.h>
 #include <stdio.h>
 #include <pwd.h>
-#include <crypt.h>
+#ifdef HAVE_CRYPT_H
+# include <crypt.h>
+#endif
 #ifdef HAVE_SHADOW_H
 #include <shadow.h>
 #endif
@@ -153,7 +155,7 @@ auth_pass (const char *passwd, struct credentials *pcred)
 	/* Try to authenticate the user.  */
 	if (pcred->passwd == NULL || *pcred->passwd == '\0')
 	  return 1; /* Failed. */
-	xpasswd = CRYPT (passwd, salt);
+	xpasswd = crypt (passwd, salt);
 	return  (!xpasswd || strcmp (xpasswd, pcred->passwd) != 0);
       }
     } /* switch (auth_type) */
