@@ -1,7 +1,26 @@
+/* Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
+
+   This file is part of GNU Inetutils.
+
+   GNU Inetutils is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2, or (at your option)
+   any later version.
+
+   GNU Inetutils is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with GNU Inetutils; see the file COPYING.  If not, write
+   to the Free Software Foundation, Inc., 51 Franklin Street,
+   Fifth Floor, Boston, MA 02110-1301 USA. */
+
 /*
- * Copyright (c) 1983, 1991, 1993, 1994, 2002
+ * Copyright (c) 1983, 1991, 1993, 1994
  *	The Regents of the University of California.  All rights reserved.
- *
+ *  
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -13,7 +32,7 @@
  * 4. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
- *
+ *  
  * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -33,12 +52,12 @@ static char sccsid[] = "@(#)inetd.c	8.4 (Berkeley) 4/13/94";
 
 /*
  * Inetd - Internet super-server
- *
+ *  
  * This program invokes all internet services as needed.  Connection-oriented
  * services are invoked each time a connection is made, by creating a process.
  * This process is passed the connection as file descriptor 0 and is expected
  * to do a getpeername to find out the source host and port.
- *
+ *  
  * Datagram oriented services are invoked when a datagram
  * arrives; a process is created and passed a pending message
  * on file descriptor 0.  Datagram servers may either connect
@@ -47,13 +66,13 @@ static char sccsid[] = "@(#)inetd.c	8.4 (Berkeley) 4/13/94";
  * processing all arriving datagrams and, eventually, timing
  * out.	 The first type of server is said to be ``multi-threaded'';
  * the second type of server ``single-threaded''.
- *
+ *  
  * Inetd uses a configuration file which is read at startup
  * and, possibly, at some later time in response to a hangup signal.
  * The configuration file is ``free format'' with fields given in the
  * order shown below.  Continuation lines for an entry must being with
  * a space or tab.  All fields must be present in each entry.
- *
+ *  
  *	service name			must be in /etc/services or must
  *					name a tcpmux service
  *	socket type			stream/dgram/raw/rdm/seqpacket
@@ -61,8 +80,8 @@ static char sccsid[] = "@(#)inetd.c	8.4 (Berkeley) 4/13/94";
  *	wait/nowait			single-threaded/multi-threaded
  *	user				user to run daemon as
  *	server program			full path name
- *	server program arguments	maximum of MAXARGS (20)
- *
+ *	server program arguments	arguments starting with argv[0]
+ *  
  * TCP services without official port numbers are handled with the
  * RFC1078-based tcpmux internal service. Tcpmux listens on port 1 for
  * requests. When a connection is made from a foreign host, the service
@@ -77,7 +96,7 @@ static char sccsid[] = "@(#)inetd.c	8.4 (Berkeley) 4/13/94";
  * special server code in them. Services that use tcpmux are "nowait"
  * because they do not have a well-known port and hence cannot listen
  * for new requests.
- *
+ *  
  * Comment lines are indicated by a `#' in column 1.
  */
 
@@ -144,22 +163,27 @@ fd_set	allsock;
 int	options;
 int	timingout;
 int	toomany = TOOMANY;
-struct	servent *sp;
 
 struct	servtab {
   char	*se_service;		/* name of service */
   int	se_socktype;		/* type of socket to use */
   char	*se_proto;		/* protocol used */
-  short	se_wait;		/* single threaded server */
+  pid_t	se_wait;		/* single threaded server */
   short	se_checked;		/* looked at during merge */
   char	*se_user;		/* user name to run as */
   struct biltin *se_bi;		/* if built-in, description */
   char	*se_server;		/* server program */
-#define	MAXARGV 20
-  char	*se_argv[MAXARGV+1];	/* program arguments */
+  char	**se_argv;	        /* program arguments */
+  size_t se_argc;               /* number of arguments */
   int	se_fd;			/* open descriptor */
   int	se_type;		/* type */
+  sa_family_t se_family;	/* address family of the socket */
+  char	se_v4mapped;		/* 1 = accept v4mapped connection, 0 = don't */
+#ifdef IPV6
+  struct sockaddr_storage se_ctrladdr;/* bound address */
+#else
   struct sockaddr_in se_ctrladdr;/* bound address */
+#endif
   int	se_count;		/* number started since se_time */
   struct	timeval se_time;	/* start of se_count */
   struct	servtab *se_next;
@@ -186,7 +210,7 @@ void		echo_stream __P((int, struct servtab *));
 void		endconfig __P((FILE *));
 struct servtab *enter __P((struct servtab *));
 void		freeconfig __P((struct servtab *));
-struct servtab *getconfigent __P((FILE *, const char *));
+struct servtab *getconfigent __P((FILE *, const char *, size_t *));
 void		machtime_dg __P((int, struct servtab *));
 void		machtime_stream __P((int, struct servtab *));
 char	       *newstr __P((const char *));
@@ -197,8 +221,6 @@ void		reapchild __P((int));
 void		retry __P((int));
 FILE           *setconfig __P((const char *));
 void		setup __P((struct servtab *));
-char	       *sskip __P((char **, FILE *, const char *));
-char	       *skip __P((char **, FILE *));
 void		tcpmux __P((int s, struct servtab *sep));
 void		set_proc_title __P ((char *, int));
 void		initring __P((void));
@@ -220,7 +242,7 @@ struct biltin {
   { "discard",	SOCK_STREAM,	1, 0,	discard_stream },
   { "discard",	SOCK_DGRAM,	0, 0,	discard_dg },
 
-  /* Return 32 bit time since 1970 */
+  /* Return 32 bit time since 1900 */
   { "time",	SOCK_STREAM,	0, 0,	machtime_stream },
   { "time",	SOCK_DGRAM,	0, 0,	machtime_dg },
 
@@ -243,6 +265,75 @@ char 	*LastArg;
 
 char **config_files;
 
+
+/* Signal handling */
+
+#if defined(HAVE_SIGACTION)
+# define SIGSTATUS sigset_t
+#else
+# define SIGSTATUS long
+#endif
+
+void
+signal_set_handler (int signo, RETSIGTYPE (*handler) ())
+{
+#if defined(HAVE_SIGACTION)
+  struct sigaction sa;
+  memset ((char *)&sa, 0, sizeof(sa));
+  sigemptyset (&sa.sa_mask);
+  sigaddset (&sa.sa_mask, signo);
+#ifdef SA_RESTART
+  sa.sa_flags = SA_RESTART;
+#endif
+  sa.sa_handler = handler;
+  sigaction (signo, &sa, NULL);
+#elif defined(HAVE_SIGVEC)
+  struct sigvec sv;
+  memset (&sv, 0, sizeof(sv));
+  sv.sv_mask = SIGBLOCK;
+  sv.sv_handler = handler;
+  sigvec (signo, &sv, NULL);
+#else /* !HAVE_SIGVEC */
+  signal (signo, handler);
+#endif /* HAVE_SIGACTION */
+}
+
+void
+signal_block (SIGSTATUS *old_status)
+{
+#ifdef HAVE_SIGACTION
+  sigset_t sigs;
+  
+  sigemptyset (&sigs);
+  sigaddset (&sigs, SIGCHLD);
+  sigaddset (&sigs, SIGHUP);
+  sigaddset (&sigs, SIGALRM);
+  sigprocmask (SIG_BLOCK, &sigs, old_status);
+#else
+  long omask = sigblock (SIGBLOCK);
+  if (old_status)
+    *old_status = omask;
+#endif
+}
+
+void
+signal_unblock (SIGSTATUS *status)
+{
+#ifdef HAVE_SIGACTION
+  if (status)
+    sigprocmask (SIG_SETMASK, status, 0);
+  else
+    {
+      sigset_t empty;
+      sigemptyset (&empty);
+      sigprocmask (SIG_SETMASK, &empty, 0);
+    }
+#else
+  sigsetmask (status ? *status : 0);
+#endif
+}
+
+
 static void
 usage (int err)
 {
@@ -359,59 +450,31 @@ main (int argc, char *argv[], char *envp[])
 
   openlog ("inetd", LOG_PID | LOG_NOWAIT, LOG_DAEMON);
 
-#if defined(HAVE_SIGACTION)
   {
-    struct sigaction sa;
-    memset ((char *)&sa, 0, sizeof(sa));
-    sigemptyset (&sa.sa_mask);
-    sigaddset (&sa.sa_mask, SIGCHLD);
-    sigaddset (&sa.sa_mask, SIGHUP);
-    sigaddset (&sa.sa_mask, SIGALRM);
-#ifdef SA_RESTART
-    sa.sa_flags = SA_RESTART;
-#endif
-    sa.sa_handler = retry;
-    sigaction (SIGALRM, &sa, (struct sigaction *)0);
-    config (0);
-    sa.sa_handler = config;
-    sigaction (SIGHUP, &sa, (struct sigaction *)0);
-    sa.sa_handler = reapchild;
-    sigaction (SIGCHLD, &sa, (struct sigaction *)0);
-    sa.sa_handler = SIG_IGN;
-    sigaction (SIGPIPE, &sa, (struct sigaction *)0);
+    FILE *fp = fopen (PATH_INETDPID, "w");
+    if (fp != NULL)
+      {
+        fprintf (fp, "%d\n", getpid ());
+        fclose (fp);
+      }
+    else
+      syslog (LOG_CRIT, "can't open %s: %s\n", PATH_INETDPID, strerror (errno));
   }
-#elif defined(HAVE_SIGVEC)
-  {
-    struct sigvec sv;
-    memset (&sv, 0, sizeof(sv));
-    sv.sv_mask = SIGBLOCK;
-    sv.sv_handler = retry;
-    sigvec (SIGALRM, &sv, (struct sigvec *)0);
-    config (0);
-    sv.sv_handler = config;
-    sigvec (SIGHUP, &sv, (struct sigvec *)0);
-    sv.sv_handler = reapchild;
-    sigvec (SIGCHLD, &sv, (struct sigvec *)0);
-    sv.sv_mask = 0L;
-    sv.sv_handler = SIG_IGN;
-    sigvec (SIGPIPE, &sv, (struct sigvec *)0);
-  }
-#else /* !HAVE_SIGVEC */
-  signal (SIGALRM, retry);
+
+  signal_set_handler (SIGALRM, retry);
   config (0);
-  signal (SIGHUP, config);
-  signal (SIGCHLD, reapchild);
-  signal (SIGPIPE, SIG_IGN);
-#endif /* HAVE_SIGACTION */
+  signal_set_handler (SIGHUP, config);
+  signal_set_handler (SIGCHLD, reapchild);
+  signal_set_handler (SIGPIPE, SIG_IGN);
 
   {
     /* space for daemons to overwrite environment for ps */
 #define	DUMMYSIZE	100
     char dummy[DUMMYSIZE];
 
-    (void)memset(dummy, 'x', DUMMYSIZE - 1);
+    memset(dummy, 'x', DUMMYSIZE - 1);
     dummy[DUMMYSIZE - 1] = '\0';
-    (void)setenv("inetd_dummy", dummy, 1);
+    setenv("inetd_dummy", dummy, 1);
   }
 
   for (;;)
@@ -421,33 +484,13 @@ main (int argc, char *argv[], char *envp[])
 
       if (nsock == 0)
 	{
-#ifdef HAVE_SIGACTION
-	  {
-	    sigset_t sigs;
-	    sigemptyset (&sigs);
-	    sigaddset (&sigs, SIGCHLD);
-	    sigaddset (&sigs, SIGHUP);
-	    sigaddset (&sigs, SIGALRM);
-	    sigprocmask (SIG_BLOCK, &sigs, 0);
-	  }
-#else
-	  (void) sigblock (SIGBLOCK);
-#endif
+	  signal_block (NULL);
 	  while (nsock == 0)
 	    sigpause (0L);
-#ifdef HAVE_SIGACTION
-	  {
-	    sigset_t empty;
-	    sigemptyset (&empty);
-	    sigprocmask (SIG_SETMASK, &empty, 0);
-	  }
-#else
-	  (void) sigsetmask (0L);
-#endif
+	  signal_unblock (NULL);
 	}
       readable = allsock;
-      if ((n = select (maxsock + 1, &readable, (fd_set *)0,
-		      (fd_set *)0, (struct timeval *)0)) <= 0)
+      if ((n = select (maxsock + 1, &readable, NULL, NULL, NULL)) <= 0)
 	{
 	  if (n < 0 && errno != EINTR)
 	    syslog (LOG_WARNING, "select: %m");
@@ -462,7 +505,7 @@ main (int argc, char *argv[], char *envp[])
 	      fprintf (stderr, "someone wants %s\n", sep->se_service);
 	    if (!sep->se_wait && sep->se_socktype == SOCK_STREAM)
 	      {
-		ctrl = accept (sep->se_fd, (struct sockaddr *)0, (int *)0);
+		ctrl = accept (sep->se_fd, NULL, NULL);
 		if (debug)
 		  fprintf (stderr, "accept, ctrl %d\n", ctrl);
 		if (ctrl < 0)
@@ -475,29 +518,19 @@ main (int argc, char *argv[], char *envp[])
 	      }
 	    else
 	      ctrl = sep->se_fd;
-#ifdef HAVE_SIGACTION
-	    {
-	      sigset_t sigs;
-	      sigemptyset (&sigs);
-	      sigaddset (&sigs, SIGCHLD);
-	      sigaddset (&sigs, SIGHUP);
-	      sigaddset (&sigs, SIGALRM);
-	      sigprocmask (SIG_BLOCK, &sigs, 0);
-	    }
-#else
-	    (void) sigblock (SIGBLOCK);
-#endif
+
+	    signal_block (NULL);
 	    pid = 0;
 	    dofork = (sep->se_bi == 0 || sep->se_bi->bi_fork);
 	    if (dofork)
 	      {
 		if (sep->se_count++ == 0)
-		  (void)gettimeofday (&sep->se_time, (struct timezone *)0);
+		  gettimeofday (&sep->se_time, NULL);
 		else if (sep->se_count >= toomany)
 		  {
 		    struct timeval now;
 
-		    (void)gettimeofday (&now, (struct timezone *)0);
+		    gettimeofday (&now, NULL);
 		    if (now.tv_sec - sep->se_time.tv_sec > CNT_INTVL)
 		      {
 			sep->se_time = now;
@@ -509,15 +542,7 @@ main (int argc, char *argv[], char *envp[])
 				"%s/%s server failing (looping), service terminated",
 				sep->se_service, sep->se_proto);
 			close_sep (sep);
-#ifdef HAVE_SIGACTION
-			{
-			  sigset_t empty;
-			  sigemptyset (&empty);
-			  sigprocmask (SIG_SETMASK, &empty, 0);
-			}
-#else
-			sigsetmask (0L);
-#endif
+			signal_unblock (NULL);
 			if (!timingout)
 			  {
 			    timingout = 1;
@@ -533,15 +558,7 @@ main (int argc, char *argv[], char *envp[])
 		syslog (LOG_ERR, "fork: %m");
 		if (!sep->se_wait && sep->se_socktype == SOCK_STREAM)
 		  close (ctrl);
-#ifdef HAVE_SIGACTION
-		{
-		  sigset_t empty;
-		  sigemptyset (&empty);
-		  sigprocmask (SIG_SETMASK, &empty, 0);
-		}
-#else
-		sigsetmask (0L);
-#endif
+		signal_unblock (NULL);
 		sleep (1);
 		continue;
 	      }
@@ -554,15 +571,7 @@ main (int argc, char *argv[], char *envp[])
 		    nsock--;
 		  }
 	      }
-#ifdef HAVE_SIGACTION
-	    {
-	      sigset_t empty;
-	      sigemptyset (&empty);
-	      sigprocmask (SIG_SETMASK, &empty, 0);
-	    }
-#else
-	    sigsetmask (0L);
-#endif
+	    signal_unblock (NULL);
 	    if (pid == 0)
 	      {
 		if (debug && dofork)
@@ -619,7 +628,7 @@ run_service (int ctrl, struct servtab *sep)
 	      _exit (1);
 	    }
 #ifdef HAVE_INITGROUPS
-	  (void) initgroups (pwd->pw_name, pwd->pw_gid);
+	  initgroups (pwd->pw_name, pwd->pw_gid);
 #endif
 	  if (setuid (pwd->pw_uid) < 0)
 	    {
@@ -643,11 +652,11 @@ reapchild (int signo)
   pid_t pid;
   struct servtab *sep;
 
-  (void)signo; /* shutup gcc */
+  signo; /* shutup gcc */
   for (;;)
     {
 #ifdef HAVE_WAIT3
-      pid = wait3 (&status, WNOHANG, (struct rusage *)0);
+      pid = wait3 (&status, WNOHANG, NULL);
 #else
       pid = wait (&status);
 #endif
@@ -678,7 +687,7 @@ config (int signo)
   struct stat stats;
   struct servtab *sep;
 
-  (void)signo; /* Shutup gcc.  */
+  signo; /* Shutup gcc.  */
 
   for (sep = servtab; sep; sep = sep->se_next)
     sep->se_checked = 0;
@@ -734,22 +743,23 @@ config (int signo)
 void
 nextconfig (const char *file)
 {
+#ifndef IPV6
+  struct servent *sp;
+#endif
   struct servtab *sep, *cp, **sepp;
   struct passwd *pwd;
   FILE * fconfig;
-#ifdef HAVE_SIGACTION
-  sigset_t sigs, osigs;
-#else
-  long omask;
-#endif
-
+  SIGSTATUS sigstatus;
+  
+  size_t line = 0;
+  
   fconfig = setconfig (file);
   if (!fconfig)
     {
       syslog (LOG_ERR, "%s: %m", file);
       return;
     }
-  while ((cp = getconfigent (fconfig, file)))
+  while ((cp = getconfigent (fconfig, file, &line)))
     {
       if ((pwd = getpwnam (cp->se_user)) == NULL)
 	{
@@ -765,17 +775,7 @@ nextconfig (const char *file)
 	  break;
       if (sep != 0)
 	{
-	  int i;
-
-#ifdef HAVE_SIGACTION
-	  sigemptyset (&sigs);
-	  sigaddset (&sigs, SIGCHLD);
-	  sigaddset (&sigs, SIGHUP);
-	  sigaddset (&sigs, SIGALRM);
-	  sigprocmask (SIG_BLOCK, &sigs, &osigs);
-#else
-	  omask = sigblock (SIGBLOCK);
-#endif
+	  signal_block (&sigstatus);
 	  /*
 	   * sep->se_wait may be holding the pid of a daemon
 	   * that we're waiting for.  If so, don't overwrite
@@ -790,13 +790,12 @@ nextconfig (const char *file)
 	    SWAP(sep->se_user, cp->se_user);
 	  if (cp->se_server)
 	    SWAP(sep->se_server, cp->se_server);
-	  for (i = 0; i < MAXARGV; i++)
-	    SWAP(sep->se_argv[i], cp->se_argv[i]);
-#ifdef HAVE_SIGACTION
-	  sigprocmask (SIG_SETMASK, &osigs, 0);
-#else
-	  sigsetmask (omask);
-#endif
+	  argcv_free(sep->se_argc, sep->se_argv);
+	  sep->se_argc = cp->se_argc;
+	  sep->se_argv = cp->se_argv;
+	  cp->se_argc = 0;
+	  cp->se_argv = NULL;
+	  signal_unblock (&sigstatus);
 	  freeconfig (cp);
 	  if (debug)
 	    print_service (file, "REDO", sep);
@@ -813,6 +812,7 @@ nextconfig (const char *file)
 	  sep->se_fd = -1;
 	  continue;
 	}
+#ifndef IPV6 /* This code is moved to setup() for IPV6.  */
       sp = getservbyname (sep->se_service, sep->se_proto);
       if (sp == 0)
 	{
@@ -828,6 +828,7 @@ nextconfig (const char *file)
 	  if (sep->se_fd >= 0)
 	    close_sep (sep);
 	}
+#endif
       if (sep->se_fd == -1)
 	setup (sep);
     }
@@ -835,15 +836,7 @@ nextconfig (const char *file)
   /*
    * Purge anything not looked at above.
    */
-#ifdef HAVE_SIGACTION
-  sigemptyset (&sigs);
-  sigaddset (&sigs, SIGCHLD);
-  sigaddset (&sigs, SIGHUP);
-  sigaddset (&sigs, SIGALRM);
-  sigprocmask (SIG_BLOCK, &sigs, &osigs);
-#else
-  omask = sigblock (SIGBLOCK);
-#endif
+  signal_block (&sigstatus);
   sepp = &servtab;
   while ((sep = *sepp))
     {
@@ -860,11 +853,7 @@ nextconfig (const char *file)
       freeconfig (sep);
       free ((char *)sep);
     }
-#ifdef HAVE_SIGACTION
-  sigprocmask (SIG_SETMASK, &osigs, 0);
-#else
-  (void) sigsetmask (omask);
-#endif
+  signal_unblock (&sigstatus);
 }
 
 void
@@ -872,7 +861,7 @@ retry (int signo)
 {
   struct servtab *sep;
 
-  (void)signo; /* shutup gcc */
+  signo; /* shutup gcc */
   timingout = 0;
   for (sep = servtab; sep; sep = sep->se_next)
     if (sep->se_fd == -1 && !ISMUX (sep))
@@ -882,32 +871,137 @@ retry (int signo)
 void
 setup (struct servtab *sep)
 {
-  int on = 1;
+  int err;
+  const int on = 1;
+#ifdef IPV6
+  const int off = 0;
+  struct addrinfo *result, hints;
+  struct protoent *proto;
 
-  if ((sep->se_fd = socket (AF_INET, sep->se_socktype, 0)) < 0)
+ tryagain:
+#endif
+  sep->se_fd = socket (sep->se_family, sep->se_socktype, 0);
+  if (sep->se_fd < 0)
     {
+#ifdef IPV6
+      /* If we don't support creating AF_INET6 sockets, create AF_INET
+	 sockets.  */
+      if (errno == EAFNOSUPPORT && sep->se_family == AF_INET6 && sep->se_v4mapped)
+	{
+	  /* Fall back to IPv6 silently.  */
+	  sep->se_family = AF_INET;
+	  goto tryagain;
+	}
+#endif
+      
       if (debug)
 	fprintf (stderr, "socket failed on %s/%s: %s\n",
 		 sep->se_service, sep->se_proto, strerror (errno));
       syslog(LOG_ERR, "%s/%s: socket: %m", sep->se_service, sep->se_proto);
       return;
     }
-#define	turnon(fd, opt) \
-setsockopt(fd, SOL_SOCKET, opt, (char *)&on, sizeof (on))
-  if (strcmp (sep->se_proto, "tcp") == 0 && (options & SO_DEBUG)
-      && turnon(sep->se_fd, SO_DEBUG) < 0)
-    syslog (LOG_ERR, "setsockopt (SO_DEBUG): %m");
-  if (turnon (sep->se_fd, SO_REUSEADDR) < 0)
-    syslog (LOG_ERR, "setsockopt (SO_REUSEADDR): %m");
-#undef turnon
-  if (bind (sep->se_fd, (struct sockaddr *)&sep->se_ctrladdr,
-	    sizeof (sep->se_ctrladdr)) < 0)
+
+#ifdef IPV6
+  /* Make sure that tcp6 etc also work.  */
+  if (strncmp (sep->se_proto, "tcp", 3) == 0)
+    proto = getprotobyname ("tcp");
+  else if (strncmp (sep->se_proto, "udp", 3) == 0)
+    proto = getprotobyname ("udp");
+  else
+    proto = getprotobyname (sep->se_proto);
+   
+  if (!proto)
     {
+      syslog (LOG_ERR, "%s: Unknown protocol", sep->se_proto);
+      close (sep->se_fd);
+      sep->se_fd = -1;
+      return;
+    }
+  
+  memset (&hints, 0, sizeof (hints));
+
+  hints.ai_flags = AI_PASSIVE;
+  hints.ai_family = sep->se_family;
+  hints.ai_socktype = sep->se_socktype;
+  hints.ai_protocol = proto->p_proto;
+
+  err = getaddrinfo (NULL, sep->se_service, &hints, &result);
+  if (err)
+    {
+      const char *errmsg;
+
+      if (err == EAI_FAMILY && sep->se_family == AF_INET6 && sep->se_v4mapped)
+	{
+	  /* Fall back to IPv6 silently.  */
+	  sep->se_family = AF_INET;
+	  close (sep->se_fd);
+	  goto tryagain;
+	}
+
+      if (err == EAI_SYSTEM)
+	errmsg = strerror (errno);
+      else
+	errmsg = gai_strerror (err);
+      
+      syslog (LOG_ERR, "%s/%s: getaddrinfo: %s",
+	      sep->se_service, sep->se_proto, errmsg);
+      
+      close (sep->se_fd);
+      sep->se_fd = -1;
+      return;
+    }
+
+  memcpy (&sep->se_ctrladdr, result->ai_addr, result->ai_addrlen);
+
+  freeaddrinfo (result);
+
+  if (sep->se_family == AF_INET6)
+    {
+      if (sep->se_v4mapped)
+	err = setsockopt (sep->se_fd, IPPROTO_IPV6, IPV6_V6ONLY,
+			  (char *)&off, sizeof(off));
+      else
+	err = setsockopt (sep->se_fd, IPPROTO_IPV6, IPV6_V6ONLY,
+			  (char *)&on, sizeof(on));
+      
+      if (err < 0)
+	syslog (LOG_ERR, "setsockopt (IPV6_V6ONLY): %m");
+    }
+#endif
+  
+  if (strncmp (sep->se_proto, "tcp", 3) == 0 && (options & SO_DEBUG))
+    {
+      err = setsockopt(sep->se_fd, SOL_SOCKET, SO_DEBUG,
+		       (char *)&on, sizeof (on));
+      if (err < 0)
+	syslog (LOG_ERR, "setsockopt (SO_DEBUG): %m");
+    }
+  
+  err = setsockopt(sep->se_fd, SOL_SOCKET, SO_REUSEADDR,
+		   (char *)&on, sizeof (on));
+  if (err < 0)
+    syslog (LOG_ERR, "setsockopt (SO_REUSEADDR): %m");
+
+  err = bind (sep->se_fd, (struct sockaddr *)&sep->se_ctrladdr,
+	      sizeof (sep->se_ctrladdr));
+  if (err < 0)
+    {
+#ifdef IPV6
+      /* If we can't bind with AF_INET6 try again with AF_INET.  */
+      if ((errno == EADDRNOTAVAIL || errno == EAFNOSUPPORT)
+	  && sep->se_family == AF_INET6 && sep->se_v4mapped)
+	{	
+	  /* Fall back to IPv6 silently.  */
+	  sep->se_family = AF_INET;
+	  close (sep->se_fd);
+	  goto tryagain;
+	}
+#endif
       if (debug)
 	fprintf (stderr, "bind failed on %s/%s: %s\n",
 		 sep->se_service, sep->se_proto, strerror (errno));
       syslog(LOG_ERR, "%s/%s: bind: %m", sep->se_service, sep->se_proto);
-      (void) close (sep->se_fd);
+      close (sep->se_fd);
       sep->se_fd = -1;
       if (!timingout)
 	{
@@ -938,7 +1032,7 @@ close_sep (struct servtab *sep)
     {
       nsock--;
       FD_CLR (sep->se_fd, &allsock);
-      (void) close (sep->se_fd);
+      close (sep->se_fd);
       sep->se_fd = -1;
     }
   sep->se_count = 0;
@@ -954,36 +1048,20 @@ struct servtab *
 enter (struct servtab *cp)
 {
   struct servtab *sep;
-#ifdef HAVE_SIGACTION
-  sigset_t sigs, osigs;
-#else
-  long omask;
-#endif
-
+  SIGSTATUS sigstatus;
+  
   sep = (struct servtab *)malloc (sizeof (*sep));
-  if (sep == (struct servtab *)0)
+  if (sep == NULL)
     {
       syslog (LOG_ERR, "Out of memory.");
       exit (-1);
     }
   *sep = *cp;
   sep->se_fd = -1;
-#ifdef HAVE_SIGACTION
-  sigemptyset (&sigs);
-  sigaddset (&sigs, SIGCHLD);
-  sigaddset (&sigs, SIGHUP);
-  sigaddset (&sigs, SIGALRM);
-  sigprocmask (SIG_BLOCK, &sigs, &osigs);
-#else
-  omask = sigblock (SIGBLOCK);
-#endif
+  signal_block (&sigstatus);
   sep->se_next = servtab;
   servtab = sep;
-#ifdef HAVE_SIGACTION
-  sigprocmask (SIG_SETMASK, &osigs, 0);
-#else
-  sigsetmask (omask);
-#endif
+  signal_unblock (&sigstatus);
   return sep;
 }
 
@@ -1005,123 +1083,200 @@ endconfig (FILE *fconfig)
 {
   if (fconfig)
     {
-      (void) fclose (fconfig);
+      fclose (fconfig);
     }
 }
 
+#define INETD_SERVICE      0   /* service name */
+#define INETD_SOCKET       1   /* socket type */
+#define INETD_PROTOCOL     2   /* protocol */
+#define INETD_WAIT         3   /* wait/nowait */
+#define INETD_USER         4   /* user */
+#define INETD_SERVER_PATH  5   /* server program path */ 
+#define INETD_SERVER_ARGS  6   /* server program arguments */
+
+#define INETD_FIELDS_MIN   6   /* Minimum number of fields in entry */
+
 struct servtab *
-getconfigent (FILE *fconfig, const char *file)
+getconfigent (FILE *fconfig, const char *file, size_t *line)
 {
   struct servtab *sep = &serv;
-  int argc;
-  char *cp, *arg;
+  size_t argc = 0, i;
+  char **argv = NULL;
+  char *cp;
   static char TCPMUX_TOKEN[] = "tcpmux/";
 #define MUX_LEN		(sizeof(TCPMUX_TOKEN)-1)
 
-more:
-  while ((cp = nextline (fconfig)) && (*cp == '#' || *cp == '\0'))
-    ;
-  if (cp == NULL)
-    return ((struct servtab *)0);
-  /*
-   * clear the static buffer, since some fields (se_ctrladdr,
-   * for example) don't get initialized here.
-   */
   memset ((caddr_t)sep, 0, sizeof *sep);
-  arg = skip (&cp, fconfig);
-  if (cp == NULL)
+  
+  while (1)
     {
-      /* got an empty line containing just blanks/tabs. */
-      goto more;
-    }
-  if (strncmp (arg, TCPMUX_TOKEN, MUX_LEN) == 0)
-    {
-      char *c = arg + MUX_LEN;
-      if (*c == '+')
+      argcv_free (argc, argv);
+      freeconfig (sep);
+      memset ((caddr_t)sep, 0, sizeof *sep);
+      
+      while ((cp = nextline (fconfig)) && (*cp == '#' || *cp == '\0'))
+	++*line;
+      ++*line;
+      if (cp == NULL)
+	return NULL;
+      
+      if (argcv_get (cp, "", &argc, &argv))
+	continue;
+      
+      if (argc < INETD_FIELDS_MIN)
 	{
-	  sep->se_type = MUXPLUS_TYPE;
-	  c++;
+	  syslog (LOG_ERR, "%s:%lu: not enough fields",
+		  file, (unsigned long) *line);
+	  continue;
+	}
+
+      if (strncmp (argv[INETD_SERVICE], TCPMUX_TOKEN, MUX_LEN) == 0)
+	{
+	  char *c = argv[INETD_SERVICE] + MUX_LEN;
+	  if (*c == '+')
+	    {
+	      sep->se_type = MUXPLUS_TYPE;
+	      c++;
+	    }
+	  else
+	    sep->se_type = MUX_TYPE;
+	  sep->se_service = newstr (c);
 	}
       else
-	sep->se_type = MUX_TYPE;
-      sep->se_service = newstr (c);
-    }
-  else
-    {
-      sep->se_service = newstr (arg);
-      sep->se_type = NORM_TYPE;
-    }
-  arg = sskip (&cp, fconfig, file);
-  if (strcmp (arg, "stream") == 0)
-    sep->se_socktype = SOCK_STREAM;
-  else if (strcmp (arg, "dgram") == 0)
-    sep->se_socktype = SOCK_DGRAM;
-  else if (strcmp (arg, "rdm") == 0)
-    sep->se_socktype = SOCK_RDM;
-  else if (strcmp (arg, "seqpacket") == 0)
-    sep->se_socktype = SOCK_SEQPACKET;
-  else if (strcmp (arg, "raw") == 0)
-    sep->se_socktype = SOCK_RAW;
-  else
-    sep->se_socktype = -1;
-  sep->se_proto = newstr (sskip (&cp, fconfig, file));
-  arg = sskip (&cp, fconfig, file);
-  sep->se_wait = strcmp (arg, "wait") == 0;
-  if (ISMUX (sep))
-    {
-      /*
-       * Silently enforce "nowait" for TCPMUX services since
-       * they don't have an assigned port to listen on.
-       */
-      sep->se_wait = 0;
+	{
+	  sep->se_service = newstr (argv[INETD_SERVICE]);
+	  sep->se_type = NORM_TYPE;
+	}
+      
+      if (strcmp (argv[INETD_SOCKET], "stream") == 0)
+	sep->se_socktype = SOCK_STREAM;
+      else if (strcmp (argv[INETD_SOCKET], "dgram") == 0)
+	sep->se_socktype = SOCK_DGRAM;
+      else if (strcmp (argv[INETD_SOCKET], "rdm") == 0)
+	sep->se_socktype = SOCK_RDM;
+      else if (strcmp (argv[INETD_SOCKET], "seqpacket") == 0)
+	sep->se_socktype = SOCK_SEQPACKET;
+      else if (strcmp (argv[INETD_SOCKET], "raw") == 0)
+	sep->se_socktype = SOCK_RAW;
+      else
+	{
+	  syslog (LOG_WARNING, "%s:%lu: bad socket type",
+		  file, (unsigned long) *line);
+	  sep->se_socktype = -1;
+	}
+      
+      sep->se_proto = newstr (argv[INETD_PROTOCOL]);
 
-      if (strcmp (sep->se_proto, "tcp"))
+#ifdef IPV6
+      /* We default to IPv6, in setup() we'll fall back to IPv4 if
+	 it doesn't work.  */
+      sep->se_family = AF_INET6;
+      sep->se_v4mapped = 1;
+      
+      if ((strncmp (sep->se_proto, "tcp", 3) == 0)
+	  || (strncmp (sep->se_proto, "udp", 3) == 0))
 	{
-	  syslog (LOG_ERR, "%s: bad protocol for tcpmux service %s",
-		  file, sep->se_service);
-	  goto more;
+	  if (sep->se_proto[3] == '6')
+	    {
+	      sep->se_family = AF_INET6;
+	      sep->se_v4mapped = 0;
+	    }
+	  else if (sep->se_proto[3] == '4')
+	    {
+	      sep->se_family = AF_INET;
+	    }
 	}
-      if (sep->se_socktype != SOCK_STREAM)
+#else
+      if ((strncmp (sep->se_proto, "tcp6", 4) == 0)
+	  || (strncmp (sep->se_proto, "udp6", 4) == 0))
 	{
-	  syslog (LOG_ERR,
-		  "%s: bad socket type for tcpmux service %s",
-		  file, sep->se_service);
-	  goto more;
+	  syslog (LOG_ERR, "%s:%lu: %s: IPv6 support isn't eneabled",
+		  file, (unsigned long) *line, sep->se_proto);
+	  continue;
 	}
-    }
-  sep->se_user = newstr (sskip (&cp, fconfig, file));
-  sep->se_server = newstr (sskip (&cp, fconfig, file));
-  if (strcmp (sep->se_server, "internal") == 0)
-    {
-      struct biltin *bi;
+      
+      sep->se_family = AF_INET;
+#endif
+      
+      if (strcmp (argv[INETD_WAIT], "wait") == 0)
+	sep->se_wait = 1;
+      else if (strcmp (argv[INETD_WAIT], "nowait") == 0)
+	sep->se_wait = 0;
+      else
+	{
+	  syslog (LOG_WARNING, "%s:%lu: bad wait type",
+		  file, (unsigned long) *line);
+	}
+  
+      if (ISMUX (sep))
+	{
+	  /*
+	   * Silently enforce "nowait" for TCPMUX services since
+	   * they don't have an assigned port to listen on.
+	   */
+	  sep->se_wait = 0;
+	  
+	  if (strncmp (sep->se_proto, "tcp", 3))
+	    {
+	      syslog (LOG_ERR, "%s:%lu: bad protocol for tcpmux service %s",
+		      file, (unsigned long) *line, sep->se_service);
+	      continue;
+	    }
+	  if (sep->se_socktype != SOCK_STREAM)
+	    {
+	      syslog (LOG_ERR,
+		      "%s:%lu: bad socket type for tcpmux service %s",
+		      file, (unsigned long) *line, sep->se_service);
+	      continue;
+	    }
+	}
+      
+      sep->se_user = newstr (argv[INETD_USER]);
+      sep->se_server = newstr (argv[INETD_SERVER_PATH]);
+      if (strcmp (sep->se_server, "internal") == 0)
+	{
+	  struct biltin *bi;
 
-      for (bi = biltins; bi->bi_service; bi++)
-	if (bi->bi_socktype == sep->se_socktype
-	    && strcmp (bi->bi_service, sep->se_service) == 0)
-	  break;
-      if (bi->bi_service == 0)
+	  for (bi = biltins; bi->bi_service; bi++)
+	    if (bi->bi_socktype == sep->se_socktype
+		&& strcmp (bi->bi_service, sep->se_service) == 0)
+	      break;
+	  if (bi->bi_service == 0)
+	    {
+	      syslog (LOG_ERR, "%s:%lu: internal service %s unknown",
+		      file, (unsigned long) *line, sep->se_service);
+	      continue;
+	    }
+	  sep->se_bi = bi;
+	  sep->se_wait = bi->bi_wait;
+	} else
+	  sep->se_bi = NULL;
+
+      sep->se_argc = argc - INETD_FIELDS_MIN + 1;
+      sep->se_argv = calloc (sep->se_argc + 1, sizeof sep->se_argv[0]);
+      if (!sep->se_argv)
 	{
-	  syslog (LOG_ERR, "internal service %s unknown", sep->se_service);
-	  goto more;
+	  syslog (LOG_ERR, "%s:%lu: Out of memory.",
+		  file, (unsigned long) *line);
+	  exit (-1);
 	}
-      sep->se_bi = bi;
-      sep->se_wait = bi->bi_wait;
-    } else
-      sep->se_bi = NULL;
-  argc = 0;
-  for (arg = skip (&cp, fconfig); cp; arg = skip (&cp, fconfig))
-    if (argc < MAXARGV)
-      sep->se_argv[argc++] = newstr (arg);
-  while (argc <= MAXARGV)
-    sep->se_argv[argc++] = NULL;
+      
+      for (i = 0; i < sep->se_argc; i++)
+	{
+	  sep->se_argv[i] = argv[INETD_SERVER_ARGS + i];
+	  argv[INETD_SERVER_ARGS + i] = 0;
+	}
+      sep->se_argv[i] = NULL;
+      break;
+    }
+  argcv_free (argc, argv);
   return sep;
 }
 
 void
 freeconfig (struct servtab *cp)
 {
-  int i;
-
   if (cp->se_service)
     free (cp->se_service);
   if (cp->se_proto)
@@ -1130,58 +1285,7 @@ freeconfig (struct servtab *cp)
     free (cp->se_user);
   if (cp->se_server)
     free (cp->se_server);
-  for (i = 0; i < MAXARGV; i++)
-    if (cp->se_argv[i])
-      free (cp->se_argv[i]);
-}
-
-
-/*
- * Safe skip - if skip returns null, log a syntax error in the
- * configuration file and exit.
- */
-char *
-sskip (char **cpp, FILE *fconfig, const char *file)
-{
-  char *cp;
-
-  cp = skip (cpp, fconfig);
-  if (cp == NULL)
-    {
-      syslog (LOG_ERR, "%s: syntax error", file);
-      exit(-1);
-    }
-  return cp;
-}
-
-char *
-skip (char **cpp, FILE *fconfig)
-{
-  char *cp = *cpp;
-  char *start;
-
-again:
-  while (*cp == ' ' || *cp == '\t')
-    cp++;
-  if (*cp == '\0')
-    {
-      int c;
-
-      c = getc (fconfig);
-      (void) ungetc (c, fconfig);
-      if (c == ' ' || c == '\t')
-	if ((cp = nextline (fconfig)))
-	  goto again;
-      *cpp = (char *)0;
-      return ((char *)0);
-    }
-  start = cp;
-  while (*cp && *cp != ' ' && *cp != '\t')
-    cp++;
-  if (*cp != '\0')
-    *cp++ = '\0';
-  *cpp = cp;
-  return start;
+  argcv_free(cp->se_argc, cp->se_argv);
 }
 
 char *
@@ -1190,7 +1294,7 @@ nextline (FILE *fd)
   char *cp;
 
   if (fgets (line, sizeof line, fd) == NULL)
-    return ((char *)0);
+    return NULL;
   cp = strchr (line, '\n');
   if (cp)
     *cp = '\0';
@@ -1212,13 +1316,31 @@ set_proc_title (char *a, int s)
 {
   int size;
   char *cp;
-  struct sockaddr_in lsin;
+#ifdef IPV6
+  struct sockaddr_storage saddr;
+#else
+  struct sockaddr_in saddr;
+#endif
   char buf[80];
 
   cp = Argv[0];
-  size = sizeof lsin;
-  if (getpeername (s, (struct sockaddr *)&lsin, &size) == 0)
-    snprintf (buf, sizeof buf, "-%s [%s]", a, inet_ntoa (lsin.sin_addr));
+  size = sizeof saddr;
+  if (getpeername (s, (struct sockaddr *)&saddr, &size) == 0)
+    {
+#ifdef IPV6
+      int err;
+      char buf2[80];
+
+      err = getnameinfo ((struct sockaddr *) &saddr, sizeof (saddr), buf2,
+			 sizeof (buf2), NULL, 0, NI_NUMERICHOST);
+      if (!err)
+	snprintf (buf, sizeof buf, "-%s [%s]", a, buf2);
+      else
+	snprintf (buf, sizeof buf, "-%s", a);
+#else
+      snprintf (buf, sizeof buf, "-%s [%s]", a, inet_ntoa (saddr.sin_addr));
+#endif
+    }
   else
     snprintf (buf, sizeof buf, "-%s", a);
   strncpy (cp, buf, LastArg - cp);
@@ -1254,13 +1376,18 @@ echo_dg (int s, struct servtab *sep)
 {
   char buffer[BUFSIZE];
   int i, size;
+#ifdef IPV6
+  struct sockaddr_storage sa;
+#else
   struct sockaddr sa;
+#endif
 
-  (void)sep;
+  sep;
   size = sizeof sa;
-  if ((i = recvfrom (s, buffer, sizeof buffer, 0, &sa, &size)) < 0)
+  i = recvfrom (s, buffer, sizeof buffer, 0, (struct sockaddr *)&sa, &size);
+  if (i < 0)
     return;
-  (void) sendto (s, buffer, i, 0, &sa, sizeof sa);
+  sendto (s, buffer, i, 0, (struct sockaddr *) &sa, sizeof sa);
 }
 
 /* ARGSUSED */
@@ -1288,8 +1415,8 @@ void
 discard_dg (int s, struct servtab *sep)
 {
   char buffer[BUFSIZE];
-  (void)sep; /* shutup gcc */
-  (void) read (s, buffer, sizeof buffer);
+  sep; /* shutup gcc */
+  read (s, buffer, sizeof buffer);
 }
 
 #include <ctype.h>
@@ -1349,12 +1476,16 @@ chargen_stream (int s, struct servtab *sep)
 void
 chargen_dg (int s, struct servtab *sep)
 {
+#ifdef IPV6
+  struct sockaddr_storage sa;
+#else
   struct sockaddr sa;
+#endif
   static char *rs;
   int len, size;
   char text[LINESIZ+2];
 
-  (void)sep; /* shutup gcc */
+  sep; /* shutup gcc */
   if (endring == 0)
     {
       initring ();
@@ -1362,7 +1493,7 @@ chargen_dg (int s, struct servtab *sep)
     }
 
   size = sizeof sa;
-  if (recvfrom (s, text, sizeof text, 0, &sa, &size) < 0)
+  if (recvfrom (s, text, sizeof text, 0, (struct sockaddr *)&sa, &size) < 0)
     return;
 
   if ((len = endring - rs) >= LINESIZ)
@@ -1376,7 +1507,7 @@ chargen_dg (int s, struct servtab *sep)
     rs = ring;
   text[LINESIZ] = '\r';
   text[LINESIZ + 1] = '\n';
-  (void) sendto (s, text, sizeof text, 0, &sa, sizeof sa);
+  sendto (s, text, sizeof text, 0, (struct sockaddr *)&sa, sizeof sa);
 }
 
 /*
@@ -1392,7 +1523,7 @@ machtime (void)
 {
   struct timeval tv;
 
-  if (gettimeofday (&tv, (struct timezone *)0) < 0)
+  if (gettimeofday (&tv, NULL) < 0)
     {
       if (debug)
 	fprintf (stderr, "Unable to get time of day\n");
@@ -1409,9 +1540,9 @@ machtime_stream (int s, struct servtab *sep)
 {
   long result;
 
-  (void)sep; /* shutup gcc */
+  sep; /* shutup gcc */
   result = machtime ();
-  (void) write (s, (char *) &result, sizeof result);
+  write (s, (char *) &result, sizeof result);
 }
 
 /* ARGSUSED */
@@ -1419,15 +1550,21 @@ void
 machtime_dg (int s, struct servtab *sep)
 {
   long result;
+#ifdef IPV6
+  struct sockaddr_storage sa;
+#else
   struct sockaddr sa;
+#endif
   int size;
 
-  (void)sep; /* shutup gcc */
+  sep; /* shutup gcc */
   size = sizeof sa;
-  if (recvfrom (s, (char *)&result, sizeof result, 0, &sa, &size) < 0)
+  if (recvfrom (s, (char *)&result, sizeof result, 0, 
+		(struct sockaddr *)&sa, &size) < 0)
     return;
   result = machtime ();
-  (void) sendto (s, (char *) &result, sizeof result, 0, &sa, sizeof sa);
+  sendto (s, (char *) &result, sizeof result, 0, 
+	  (struct sockaddr *)&sa, sizeof sa);
 }
 
 /* ARGSUSED */
@@ -1438,11 +1575,11 @@ daytime_stream (int s, struct servtab *sep)
   char buffer[256];
   time_t lclock;
 
-  (void)sep; /*shutup gcc*/
+  sep; /*shutup gcc*/
   lclock = time ((time_t *) 0);
 
-  (void) sprintf (buffer, "%.24s\r\n", ctime(&lclock));
-  (void) write (s, buffer, strlen(buffer));
+  sprintf (buffer, "%.24s\r\n", ctime(&lclock));
+  write (s, buffer, strlen(buffer));
 }
 
 /* ARGSUSED */
@@ -1452,17 +1589,21 @@ daytime_dg(int s, struct servtab *sep)
 {
   char buffer[256];
   time_t lclock;
+#ifdef IPV6
+  struct sockaddr_storage sa;
+#else
   struct sockaddr sa;
+#endif
   int size;
 
-  (void)sep; /* shutup gcc */
+  sep; /* shutup gcc */
   lclock = time ((time_t *) 0);
 
   size = sizeof sa;
-  if (recvfrom (s, buffer, sizeof buffer, 0, &sa, &size) < 0)
+  if (recvfrom (s, buffer, sizeof buffer, 0, (struct sockaddr *)&sa, &size) < 0)
     return;
-  (void) sprintf (buffer, "%.24s\r\n", ctime (&lclock));
-  (void) sendto (s, buffer, strlen(buffer), 0, &sa, sizeof sa);
+  sprintf (buffer, "%.24s\r\n", ctime (&lclock));
+  sendto (s, buffer, strlen(buffer), 0, (struct sockaddr *)&sa, sizeof sa);
 }
 
 /*
@@ -1509,7 +1650,7 @@ fd_getline (int fd, char *buf, int len)
 
 #define MAX_SERV_LEN	(256+2)		/* 2 bytes for \r\n */
 
-#define strwrite(fd, buf)	(void) write(fd, buf, sizeof(buf)-1)
+#define strwrite(fd, buf)	write(fd, buf, sizeof(buf)-1)
 
 void
 tcpmux(int s, struct servtab *sep)
@@ -1538,7 +1679,7 @@ tcpmux(int s, struct servtab *sep)
 	{
 	  if (!ISMUX (sep))
 	    continue;
-	  (void)write (s, sep->se_service, strlen (sep->se_service));
+	  write (s, sep->se_service, strlen (sep->se_service));
 	  strwrite (s, "\r\n");
 	}
       _exit (1);
