@@ -1,4 +1,4 @@
-/* Copyright (C) 1998,2001, 2002 Free Software Foundation, Inc.
+/* Copyright (C) 1998,2001, 2002, 2005 Free Software Foundation, Inc.
 
    This file is part of GNU Inetutils.
 
@@ -13,9 +13,9 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Inetutils; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA. */
+   along with GNU Inetutils; see the file COPYING.  If not, write
+   to the Free Software Foundation, Inc., 51 Franklin Street,
+   Fifth Floor, Boston, MA 02110-1301 USA. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -39,16 +39,18 @@
 #include <netdb.h>
 #include <unistd.h>
 #include <stdlib.h>
+#include <stdbool.h>
 #include <string.h>
 #include <stdio.h>
 #include <ctype.h>
 #include <errno.h>
 #include <limits.h>
 
-#include "getopt.h"
+#include <getopt.h>
 #include <icmp.h>
 #include <ping.h>
-#include <ping_impl.h>
+#include "ping_common.h"
+#include "ping_impl.h"
 
 static char short_options[] = "VLhc:dfi:l:np:qRrs:t:v";
 static struct option long_options[] = 
@@ -80,46 +82,24 @@ static struct option long_options[] =
   {NULL,      no_argument, NULL, 0}
 };
 
-extern int ping_echo __P ((int argc, char **argv));
-extern int ping_timestamp __P ((int argc, char **argv));
-extern int ping_address __P ((int argc, char **argv));
-extern int ping_router __P ((int argc, char **argv));
+extern int ping_echo (int argc, char **argv);
+extern int ping_timestamp (int argc, char **argv);
+extern int ping_address (int argc, char **argv);
+extern int ping_router (int argc, char **argv);
 
 PING *ping;
 u_char *data_buffer;
 size_t data_length = PING_DATALEN;
 unsigned options;
 unsigned long preload = 0;
-int (*ping_type) __P ((int argc, char **argv)) = ping_echo;
+int (*ping_type) (int argc, char **argv) = ping_echo;
 
 
 static void show_usage (void);
-static void show_license (void);
-static void decode_pattern (const char *text, int *pattern_len, 
-		           u_char *pattern_data);
 static void decode_type (const char *optarg);
-static void init_data_buffer (u_char *pat, int len);
 static int send_echo (PING *ping);
 
-static size_t
-ping_cvt_number (const char *optarg, size_t maxval)
-{
-  char *p;
-  size_t n;
-
-  n = strtoul (optarg, &p, 0);
-  if (*p)
-    {
-      fprintf (stderr, "Invalid value (`%s' near `%s')\n", optarg, p);
-      exit (1);
-    }
-  if (maxval && n > maxval)
-    {
-      fprintf (stderr, "Option value too big: %s\n", optarg);
-      exit (1);
-    }
-  return n;
-}
+#define MIN_USER_INTERVAL (200000/PING_PRECISION)
 
 int
 main (int argc, char **argv)
@@ -130,17 +110,14 @@ main (int argc, char **argv)
   u_char pattern[16];
   int pattern_len = 16;
   u_char *patptr = NULL;
-  int is_root = getuid () == 0;
+  bool is_root = false;
 
-  if ((ping = ping_init (ICMP_ECHO, getpid ())) == NULL)
-    {
-      fprintf (stderr, "can't init ping: %s\n", strerror (errno));
-      exit (1);
-    }
-  ping_set_sockopt (ping, SO_BROADCAST, (char *)&one, sizeof (one));
-
-  /* Reset root privileges */
-  setuid (getuid ());
+  size_t count = 0;
+  int socket_type = 0;
+  size_t interval = 0;
+ 
+  if (getuid () == 0)
+    is_root = true;
 
   /* Parse command line */
   while ((c = getopt_long (argc, argv, short_options, long_options, NULL))
@@ -150,7 +127,7 @@ main (int argc, char **argv)
 	{
 	case 'V':
 	  printf ("ping - %s %s\n", PACKAGE_NAME, PACKAGE_VERSION);
-	  printf ("Copyright (C) 1998,2001 Free Software Foundation, Inc.\n");
+	  printf ("Copyright (C) 2005 Free Software Foundation, Inc.\n");
 	  printf ("%s comes with ABSOLUTELY NO WARRANTY.\n", PACKAGE_NAME);
 	  printf ("You may redistribute copies of %s\n", PACKAGE_NAME);
 	  printf ("under the terms of the GNU General Public License.\n");
@@ -169,20 +146,37 @@ main (int argc, char **argv)
 	  break;
 	  
 	case 'c':
-	  ping_set_count (ping, ping_cvt_number (optarg, 0));
+	  count = ping_cvt_number (optarg, 0, 0);
 	  break;
 	  
 	case 'd':
-	  ping_set_sockopt (ping, SO_DEBUG, &one, sizeof (one));
+	  socket_type = SO_DEBUG;
 	  break;
 	  
 	case 'r':
-	  ping_set_sockopt (ping, SO_DONTROUTE, &one, sizeof (one));
+	  socket_type = SO_DONTROUTE;
 	  break;
 	  
 	case 'i':
-	  options |= OPT_INTERVAL;
-	  ping_set_interval (ping, ping_cvt_number (optarg, 0));
+	  {
+	    double v;
+
+	    v = strtod (optarg, &p);
+	    if (*p)
+	      {
+		fprintf (stderr, "Invalid value (`%s' near `%s')\n",
+			 optarg, p);
+		exit (1);
+	      }
+	    
+	    options |= OPT_INTERVAL;
+	    interval = v * PING_PRECISION;
+	    if (!is_root && interval < MIN_USER_INTERVAL)
+	      {
+		fprintf (stderr, "Option value too small: %s\n", optarg);
+		exit (1);
+	      }
+	  }
 	  break;
 	  
 	case 'p':
@@ -191,7 +185,7 @@ main (int argc, char **argv)
 	  break;
 	  
  	case 's':
-	  data_length = ping_cvt_number (optarg, PING_MAX_DATALEN);
+	  data_length = ping_cvt_number (optarg, PING_MAX_DATALEN, 1);
  	  break;
 	  
 	case 'n':
@@ -225,7 +219,7 @@ main (int argc, char **argv)
 	  break;
 
 	case 'f':
-	  if (!is_root)
+	  if (is_root == false)
 	    {
 	      fprintf (stderr, "ping: option not allowed: --flood\n");
 	      exit (1);
@@ -247,10 +241,20 @@ main (int argc, char **argv)
 	  break;
 	  
 	case ICMP_ADDRESS:
+	  if (!is_root)
+	    {
+	      fprintf (stderr, "ping: option not allowed: --address\n");
+	      exit (1);
+	    }
 	  decode_type ("address");
 	  break;
 	  
 	case ICMP_ROUTERDISCOVERY:
+	  if (!is_root)
+	    {
+	      fprintf (stderr, "ping: option not allowed: --router\n");
+	      exit (1);
+	    }
 	  decode_type ("router");
 	  break;
 	  
@@ -268,41 +272,32 @@ main (int argc, char **argv)
       exit (0);
     }
 
+  ping = ping_init (ICMP_ECHO, getpid ());
+  if (ping == NULL)
+    {
+      fprintf (stderr, "can't init ping: %s\n", strerror (errno));
+      exit (1);
+    }
+  ping_set_sockopt (ping, SO_BROADCAST, (char *)&one, sizeof (one));
+  
+  /* Reset root privileges */
+  setuid (getuid ());
+
+  if (count != 0)
+    ping_set_count (ping, count);
+  
+  if (socket_type != 0)
+    ping_set_sockopt (ping, socket_type, &one, sizeof (one));
+
+  if (options & OPT_INTERVAL)
+    ping_set_interval (ping, interval);
+
   init_data_buffer (patptr, pattern_len);
 
   return (*ping_type)(argc, argv);
 }
 
-void
-init_data_buffer (u_char *pat, int len)
-{
-  int i = 0;
-  u_char *p;
 
-  if (data_length == 0)
-    return;
-  data_buffer = malloc (data_length);
-  if (!data_buffer)
-    {
-      fprintf (stderr, "ping: out of memory\n");
-      exit (1);
-    }
-  if (pat)
-    {
-      for (p = data_buffer; p < data_buffer + data_length; p++)
-	{
-	  *p = pat[i];
-	  if (i++ >= len)
-	    i = 0;
-	}
-    }
-  else
-    {
-      for (i = 0; i < data_length; i++)
-	data_buffer[i] = i;
-    }
-}
-  
 
 void
 decode_type (const char *optarg)
@@ -324,23 +319,6 @@ decode_type (const char *optarg)
     }
 }
 
-void
-decode_pattern (const char *text, int *pattern_len, u_char *pattern_data)
-{
-  int i, c, off;
-
-  for (i = 0; *text && i < *pattern_len; i++)
-    {
-      if (sscanf (text, "%2x%n", &c, &off) != 1)
-	{
-	  fprintf (stderr, "ping: error in pattern near %s\n", text);
-	  exit (1);
-	}
-      text += off;
-    }
-  *pattern_len = i;
-}
-
 int volatile stop = 0;
 
 RETSIGTYPE
@@ -373,17 +351,14 @@ ping_run (PING *ping, int (*finish)())
       intvl.tv_usec = 10000;
     }
   else
-    {
-      intvl.tv_sec = ping->ping_interval;
-      intvl.tv_usec = 0;
-    }
+    PING_SET_INTERVAL(intvl, ping->ping_interval);
   
   gettimeofday (&last, NULL);
   send_echo (ping);
 
   while (!stop)
     {
-      int n, len;
+      int n;
       
       FD_ZERO (&fdset);
       FD_SET (ping->ping_fd, &fdset);
@@ -405,7 +380,8 @@ ping_run (PING *ping, int (*finish)())
       if (timeout.tv_sec < 0)
 	timeout.tv_sec = timeout.tv_usec = 0;
       
-      if ((n = select (fdmax, &fdset, NULL, NULL, &timeout)) < 0)
+      n = select (fdmax, &fdset, NULL, NULL, &timeout);
+      if (n < 0)
 	{
 	  if (errno != EINTR)
 	    perror ("select");
@@ -429,9 +405,7 @@ ping_run (PING *ping, int (*finish)())
 	    {
 	      send_echo (ping);
 	      if (!(options & OPT_QUIET) && options & OPT_FLOOD)
-		{
-		  putchar ('.');
-		}
+		putchar ('.');
 	    }
 	  else if (finishing)
 	    break;
@@ -483,8 +457,8 @@ ping_finish ()
 	printf ("-- somebody's printing up packets!");
       else
 	printf ("%d%% packet loss",
-	       (int) (((ping->ping_num_xmit - ping->ping_num_recv) * 100) /
-		      ping->ping_num_xmit));
+		(int) (((ping->ping_num_xmit - ping->ping_num_recv) * 100) /
+		       ping->ping_num_xmit));
 
     }
   printf ("\n");
@@ -503,9 +477,9 @@ Informational options:\n\
   -V, --version      output version information and exit\n\
 Options controlling ICMP request types:\n\
   --echo             Send ICMP_ECHO requests (default)\n\
-  --address          Send ICMP_ADDRESS packets\n\
+* --address          Send ICMP_ADDRESS packets\n\
   --timestamp        Send ICMP_TIMESTAMP packets\n\
-  --router           Send ICMP_ROUTERDISCOVERY packets\n\
+* --router           Send ICMP_ROUTERDISCOVERY packets\n\
 Options valid for all request types:\n\
   -c, --count N      stop after sending N packets\n\
   -d, --debug        set the SO_DEBUG option\n\
@@ -524,27 +498,6 @@ Options valid for --echo requests:\n\
 \n\
 Options marked with an * are available only to super-user\n\
 \n\
-report bugs to " PACKAGE_BUGREPORT ".\n\
+Report bugs to <" PACKAGE_BUGREPORT ">.\n\
 ");
 }
-
-void
-show_license (void)
-{
-  static char license_text[] =
-"   This program is free software; you can redistribute it and/or modify\n"
-"   it under the terms of the GNU General Public License as published by\n"
-"   the Free Software Foundation; either version 2, or (at your option)\n"
-"   any later version.\n"
-"\n"
-"   This program is distributed in the hope that it will be useful,\n"
-"   but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
-"   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
-"   GNU General Public License for more details.\n"
-"\n"
-"   You should have received a copy of the GNU General Public License\n"
-"   along with this program; if not, write to the Free Software\n"
-"   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n";
-  printf ("%s", license_text);
-}
-
