@@ -104,25 +104,6 @@ static char sccsid[] = "@(#)syslogd.c	8.3 (Berkeley) 4/4/94";
 #include <unistd.h>
 #include <getopt.h>
 
-#ifdef HAVE_UTMP_H
-# include <utmp.h>
-#endif
-
-#ifdef UTMPX
-# ifdef HAVE_UTMPX_H
-#  include <utmpx.h>
-# endif
-typedef struct utmpx UTMP;
-# define SETUTENT() setutxent()
-# define GETUTENT() getutxent()
-# define ENDUTENT() endutxent()
-#else
-typedef struct utmp UTMP;
-# define SETUTENT() setutent()
-# define GETUTENT() getutent()
-# define ENDUTENT() endutent()
-#endif
-
 #ifdef HAVE_STDARG_H
 # include <stdarg.h>
 #else
@@ -135,6 +116,9 @@ typedef struct utmp UTMP;
 # include <syslog-int.h>
 #endif
 
+#include <libinetutils.h>
+#include <readutmp.h>
+
 /* A mask of all facilities mentioned explicitly in the configuration file
  *
  * This is used to support a virtual facility "**" that covers all the rest,
@@ -275,44 +259,43 @@ time_t  now;                    /* Time use for mark and forward supending.  */
 int force_sync;                 /* GNU/Linux behaviour to sync on every line.
 				   This off by default. Set to 1 to enable.  */
 
-extern char *localhost __P ((void));
-extern int waitdaemon __P ((int nochdir, int noclose, int maxwait));
-extern char *__progname;
+char *program_name;
+extern int waitdaemon (int nochdir, int noclose, int maxwait);
 
-void cfline __P ((const char *, struct filed *));
-const char *cvthname __P ((struct sockaddr_in *));
-int decode __P ((const char *, CODE *));
-void die __P ((int));
-void domark __P ((int));
-void fprintlog __P ((struct filed *, const char *, int, const char *));
-void init __P ((int));
-void logerror __P ((const char *));
-void logmsg __P ((int, const char *, const char *, int));
-void printline __P ((const char *, const char *));
-void printsys __P ((const char *));
-char *ttymsg __P ((struct iovec *, int, char *, int));
-static void usage __P ((int));
-void wallmsg __P ((struct filed *, struct iovec *));
-char **crunch_list __P ((char **oldlist, char *list));
-char *textpri __P ((int pri));
-void dbg_toggle __P ((int));
-static void dbg_printf __P ((const char *, ...));
-void trigger_restart __P ((int));
-static void add_funix __P ((const char *path));
-static int create_unix_socket __P ((const char *path));
-static int create_inet_socket __P ((void));
+void cfline (const char *, struct filed *);
+const char *cvthname (struct sockaddr_in *);
+int decode (const char *, CODE *);
+void die (int);
+void domark (int);
+void fprintlog (struct filed *, const char *, int, const char *);
+void init (int);
+void logerror (const char *);
+void logmsg (int, const char *, const char *, int);
+void printline (const char *, const char *);
+void printsys (const char *);
+char *ttymsg (struct iovec *, int, char *, int);
+static void usage (int);
+void wallmsg (struct filed *, struct iovec *);
+char **crunch_list (char **oldlist, char *list);
+char *textpri (int pri);
+void dbg_toggle (int);
+static void dbg_printf (const char *, ...);
+void trigger_restart (int);
+static void add_funix (const char *path);
+static int create_unix_socket (const char *path);
+static int create_inet_socket (void);
 
 static void
 usage (int err)
 {
   if (err != 0)
     {
-      fprintf (stderr, "Usage: %s [OPTION] ...\n", __progname);
-      fprintf (stderr, "Try `%s --help' for more information.\n", __progname);
+      fprintf (stderr, "Usage: %s [OPTION] ...\n", program_name);
+      fprintf (stderr, "Try `%s --help' for more information.\n", program_name);
     }
   else
     {
-      fprintf (stdout, "Usage: %s [OPTION] ...\n", __progname);
+      fprintf (stdout, "Usage: %s [OPTION] ...\n", program_name);
       puts ("Log system messages.\n\n\
   -f, --rcfile=FILE  Override configuration file (default: " PATH_LOGCONF ")\n\
       --pidfile=FILE Override pidfile (default: " PATH_LOGPID ")\n\
@@ -381,9 +364,7 @@ main (int argc, char *argv[])
   struct pollfd *fdarray;
   unsigned long nfds = 0;
 
-#ifndef HAVE___PROGNAME
-  __progname = argv[0];
-#endif
+  program_name = argv[0];
 
   /* Initiliaze PATH_LOG as the first element of the unix sockets array.  */
   add_funix (PATH_LOG);
@@ -490,7 +471,7 @@ main (int argc, char *argv[])
       if (ppid < 0)
 	{
 	  fprintf (stderr, "%s: could not become daemon: %s\n",
-		   __progname, strerror (errno));
+		   program_name, strerror (errno));
 	  exit (1);
 	}
     }
@@ -505,7 +486,7 @@ main (int argc, char *argv[])
   if (LocalHostName == NULL)
     {
       fprintf (stderr, "%s: can't get local host name: %s\n",
-	       __progname, strerror (errno));
+	       program_name, strerror (errno));
       exit (2);
     }
   /* Get the domainname.  */
@@ -541,19 +522,19 @@ main (int argc, char *argv[])
   consfile.f_type = F_CONSOLE;
   consfile.f_un.f_fname = strdup (ctty);
 
-  (void) signal (SIGTERM, die);
-  (void) signal (SIGINT, Debug ? die : SIG_IGN);
-  (void) signal (SIGQUIT, Debug ? die : SIG_IGN);
-  (void) signal (SIGALRM, domark);
-  (void) signal (SIGUSR1, Debug ? dbg_toggle : SIG_IGN);
-  (void) alarm (TIMERINTVL);
+  signal (SIGTERM, die);
+  signal (SIGINT, Debug ? die : SIG_IGN);
+  signal (SIGQUIT, Debug ? die : SIG_IGN);
+  signal (SIGALRM, domark);
+  signal (SIGUSR1, Debug ? dbg_toggle : SIG_IGN);
+  alarm (TIMERINTVL);
 
   /* We add  2 = 1(klog) + 1(inet), even if they may be not use.  */
   fdarray = (struct pollfd *) malloc ((nfunix + 2) * sizeof (*fdarray));
   if (fdarray == NULL)
     {
       fprintf (stderr, "%s: can't allocate fd table: %s\n",
-	       __progname, strerror (errno));
+	       program_name, strerror (errno));
       exit (2);
     }
 
@@ -616,7 +597,7 @@ main (int argc, char *argv[])
   if (fp != NULL)
     {
       fprintf (fp, "%d\n", getpid ());
-      (void) fclose (fp);
+      fclose (fp);
     }
 
   dbg_printf ("off & running....\n");
@@ -799,7 +780,7 @@ add_funix (const char *name)
   if (funix == NULL)
     {
       fprintf (stderr, "%s: cannot allocate space for unix sockets: %s\n",
-	       __progname, strerror (errno));
+	       program_name, strerror (errno));
       exit (1);
     }
   funix[nfunix].name = name;
@@ -817,11 +798,11 @@ create_unix_socket (const char *path)
   if (path[0] == '\0')
     return -1;
 
-  (void) unlink (path);
+  unlink (path);
 
   memset (&sunx, 0, sizeof (sunx));
   sunx.sun_family = AF_UNIX;
-  (void) strncpy (sunx.sun_path, path, sizeof (sunx.sun_path));
+  strncpy (sunx.sun_path, path, sizeof (sunx.sun_path));
   fd = socket (AF_UNIX, SOCK_DGRAM, 0);
   if (fd < 0 || bind(fd, (struct sockaddr *) &sunx, SUN_LEN (&sunx)) < 0
       || chmod(path, 0666) < 0)
@@ -895,7 +876,7 @@ crunch_list (char **oldlist, char *list)
   if (oldlist == NULL)
     {
       fprintf (stderr, "%s: can't allocate memory: %s",
-	       __progname, strerror (errno));
+	       program_name, strerror (errno));
       exit (1);
     }
 
@@ -912,7 +893,7 @@ crunch_list (char **oldlist, char *list)
       if (oldlist[count] == NULL)
 	{
 	  fprintf (stderr, "%s: can't allocate memory: %s",
-		   __progname, strerror (errno));
+		   program_name, strerror (errno));
 	  exit (1);
 	}
       strncpy (oldlist[count], p, q - p);
@@ -924,7 +905,7 @@ crunch_list (char **oldlist, char *list)
   if (oldlist[count] == NULL)
     {
       fprintf (stderr, "%s: can't allocate memory: %s",
-	       __progname, strerror (errno));
+	       program_name, strerror (errno));
       exit(1);
     }
   strcpy (oldlist[count], p);
@@ -1081,7 +1062,7 @@ logmsg (int pri, const char *msg, const char *from, int flags)
       msg[9] != ':' || msg[12] != ':' || msg[15] != ' ')
     flags |= ADDDATE;
 
-  (void) time (&now);
+  time (&now);
   if (flags & ADDDATE)
     timestamp = ctime (&now) + 4;
   else
@@ -1107,12 +1088,12 @@ logmsg (int pri, const char *msg, const char *from, int flags)
       if (f->f_file >= 0)
 	{
 	  fprintlog (f, from, flags, msg);
-	  (void) close (f->f_file);
+	  close (f->f_file);
 	}
 #ifdef HAVE_SIGACTION
       sigprocmask (SIG_SETMASK, &osigs, 0);
 #else
-      (void) sigsetmask (omask);
+      sigsetmask (omask);
 #endif
       return;
     }
@@ -1133,7 +1114,7 @@ logmsg (int pri, const char *msg, const char *from, int flags)
       if ((flags & MARK) == 0 && msglen == f->f_prevlen && f->f_prevhost
 	  && !strcmp (msg, f->f_prevline) && !strcmp (from, f->f_prevhost))
 	{
-	  (void) strncpy (f->f_lasttime, timestamp,
+	  strncpy (f->f_lasttime, timestamp,
 			  sizeof (f->f_lasttime) - 1);
 	  f->f_prevcount++;
 	  dbg_printf ("msg repeated %d times, %ld sec of %d\n",
@@ -1154,7 +1135,7 @@ logmsg (int pri, const char *msg, const char *from, int flags)
 	  if (f->f_prevcount)
 	    fprintlog (f, from, 0, (char *)NULL);
 	  f->f_repeatcount = 0;
-	  (void) strncpy (f->f_lasttime, timestamp,
+	  strncpy (f->f_lasttime, timestamp,
 			  sizeof (f->f_lasttime) - 1);
 	  if (f->f_prevhost)
 	    free (f->f_prevhost);
@@ -1163,7 +1144,7 @@ logmsg (int pri, const char *msg, const char *from, int flags)
 	    {
 	      f->f_prevlen = msglen;
 	      f->f_prevpri = pri;
-	      (void) strcpy (f->f_prevline, msg);
+	      strcpy (f->f_prevline, msg);
 	      fprintlog (f, from, flags, (char *)NULL);
 	    }
 	  else
@@ -1177,7 +1158,7 @@ logmsg (int pri, const char *msg, const char *from, int flags)
 #ifdef HAVE_SIGACTION
   sigprocmask (SIG_SETMASK, &osigs, 0);
 #else
-  (void) sigsetmask (omask);
+  sigsetmask (omask);
 #endif
 }
 
@@ -1300,7 +1281,7 @@ fprintlog (struct filed *f, const char *from, int flags, const char *msg)
     dbg_printf (" %s\n", f->f_un.f_forw.f_hname);
     if (strcasecmp (from, LocalHostName) && NoHops)
       dbg_printf ("Not forwarding remote message.\n");
-    else if (!NoForward)
+    else if (NoForward)
       dbg_printf ("Not forwarding because forwarding is disabled.\n");
     else if (finet < 0)
       dbg_printf ("Not forwarding because of invalid inet fd.\n");
@@ -1359,7 +1340,7 @@ fprintlog (struct filed *f, const char *from, int flags, const char *msg)
 	  if (f->f_type == F_PIPE && e == EAGAIN)
 	    break;
 
-	  (void) close (f->f_file);
+	  close (f->f_file);
 	  /* Check for errors on TTY's due to loss of tty. */
 	  if ((e == EIO || e == EBADF)
 	      && (f->f_type == F_TTY || f->f_type == F_CONSOLE))
@@ -1381,7 +1362,7 @@ fprintlog (struct filed *f, const char *from, int flags, const char *msg)
 	    }
 	}
       else if ((flags & SYNC_FILE) && !(f->f_flags & OMIT_SYNC))
-	(void) fsync (f->f_file);
+	fsync (f->f_file);
       break;
 
     case F_USERS:
@@ -1404,7 +1385,8 @@ void
 wallmsg (struct filed *f, struct iovec *iov)
 {
   static int reenter;	/* avoid calling ourselves */
-  UTMP *utp;
+  STRUCT_UTMP *utmpbuf, *utp;
+  size_t utmp_count;
   int i;
   char *p;
   char line[sizeof(utp->ut_line) + 1];
@@ -1412,17 +1394,10 @@ wallmsg (struct filed *f, struct iovec *iov)
   if (reenter++)
     return;
 
-  SETUTENT();
+  read_utmp (PATH_UTMP, &utmp_count, &utmpbuf, READ_UTMP_CHECK_PIDS);
 
-  while ((utp = GETUTENT ()) != NULL)
+  for (utp = utmpbuf; utp < utmpbuf + utmp_count; utp++)
     {
-      /* We only want interrested to send to actual
-	 process, USER_PROCESS where somebody might listen. */
-      if (utp->ut_user[0] == '\0'
-	  || utp->ut_line[0] == '\0'
-	  || utp->ut_type != USER_PROCESS)
-	continue;
-
       strncpy (line, utp->ut_line, sizeof (utp->ut_line));
       line[sizeof (utp->ut_line)] = '\0';
       if (f->f_type == F_WALL)
@@ -1440,8 +1415,8 @@ wallmsg (struct filed *f, struct iovec *iov)
 	}
       /* Should we send the message to this user? */
       for (i = 0; i < f->f_un.f_user.f_nusers; i++)
-	if (!strncmp (f->f_un.f_user.f_unames[i], utp->ut_user,
-		      sizeof (utp->ut_user)))
+	if (!strncmp (f->f_un.f_user.f_unames[i], UT_USER (utp),
+		      sizeof (UT_USER (utp))))
 	  {
 	    p = ttymsg (iov, IOVCNT, line, TTYMSGTIME);
 	    if (p != NULL)
@@ -1452,7 +1427,7 @@ wallmsg (struct filed *f, struct iovec *iov)
 	    break;
 	  }
     }
-  ENDUTENT ();
+  free (utmpbuf);
   reenter = 0;
 }
 
@@ -1524,7 +1499,7 @@ domark (int signo)
 {
   struct filed *f;
 
-  (void)signo; /* Ignored.  */
+  signo; /* Ignored.  */
   now = time ((time_t *) NULL);
   if (MarkInterval > 0)
     {
@@ -1547,7 +1522,7 @@ domark (int signo)
 	  BACKOFF (f);
 	}
     }
-  (void) alarm (TIMERINTVL);
+  alarm (TIMERINTVL);
 }
 
 /* Print syslogd errors some place.  */
@@ -1557,16 +1532,16 @@ logerror (const char *type)
   char buf[100];
 
   if (errno)
-    (void) snprintf (buf, sizeof (buf),
+    snprintf (buf, sizeof (buf),
 		     "syslogd: %s: %s", type, strerror(errno));
   else
-    (void) snprintf (buf, sizeof (buf), "syslogd: %s", type);
+    snprintf (buf, sizeof (buf), "syslogd: %s", type);
   errno = 0;
   dbg_printf ("%s\n", buf);
   logmsg (LOG_SYSLOG | LOG_ERR, buf, LocalHostName, ADDDATE);
 }
 
-void
+RETSIGTYPE
 die (int signo)
 {
   struct filed *f;
@@ -1584,7 +1559,7 @@ die (int signo)
   Initialized = was_initialized;
   if (signo)
     {
-      dbg_printf ("%s: exiting on signal %d\n", __progname, signo);
+      dbg_printf ("%s: exiting on signal %d\n", program_name, signo);
       snprintf (buf, sizeof (buf), "exiting on signal %d", signo);
       errno = 0;
       logerror (buf);
@@ -1598,7 +1573,7 @@ die (int signo)
       {
 	close (funix[i].fd);
 	if (funix[i].name)
-	  (void) unlink (funix[i].name);
+	  unlink (funix[i].name);
       }
 
   if (finet >= 0)
@@ -1608,7 +1583,7 @@ die (int signo)
 }
 
 /* INIT -- Initialize syslogd from configuration table.  */
-void
+RETSIGTYPE
 init (int signo)
 {
   FILE *cf;
@@ -1623,7 +1598,6 @@ init (int signo)
   int cont_line = 0;
   struct servent *sp;
 
-  (void) signo;  /* Ignored.  */
   dbg_printf ("init\n");
   sp = getservbyname ("syslog", "udp");
   if (sp == NULL)
@@ -1649,7 +1623,7 @@ init (int signo)
 	case F_TTY:
 	case F_CONSOLE:
 	case F_PIPE:
-	  (void) close (f->f_file);
+	  close (f->f_file);
 	  break;
 	}
       next = f->f_next;
@@ -1766,7 +1740,7 @@ init (int signo)
     }
 
   /* Close the configuration file.  */
-  (void) fclose (cf);
+  fclose (cf);
   free (cbuf);
 
   Initialized = 1;
@@ -2055,12 +2029,11 @@ decode (const char *name, CODE *codetab)
   return -1;
 }
 
-void
-dbg_toggle(int signo)
+RETSIGTYPE
+dbg_toggle(int signo ARG_UNUSED)
 {
   int dbg_save = dbg_output;
 
-  (void) signo;  /* Ignored.  */
   dbg_output = 1;
   dbg_printf ("Switching dbg_output to %s.\n",
 	      dbg_save == 0 ? "true" : "false");
@@ -2094,10 +2067,9 @@ dbg_printf (const char *fmt, ...)
    not being doing this during a signal handler.  Instead we simply
    set a flag variable which will tell the main loop to go through a
    restart.  */
-void
-trigger_restart (int signo)
+RETSIGTYPE
+trigger_restart (int signo ARG_UNUSED)
 {
-  (void) signo;  /* Ignored.  */
   restart = 1;
 }
 
