@@ -28,8 +28,7 @@
  */
 
 #ifndef lint
-static char copyright[] =
-"@(#) Copyright (c) 1983, 1993\n\
+static char copyright[] = "@(#) Copyright (c) 1983, 1993\n\
 	The Regents of the University of California.  All rights reserved.\n";
 #endif /* not lint */
 
@@ -40,9 +39,12 @@ static char sccsid[] = "@(#)talk.c	8.1 (Berkeley) 6/6/93";
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
+#include <getopt.h>
 
 #include "talk.h"
 
+void usage (void);
+
 /*
  * talk:	A visual form of write. Using sockets, a two way
  *		connection is set up between the two people talking.
@@ -57,22 +59,61 @@ static char sccsid[] = "@(#)talk.c	8.1 (Berkeley) 6/6/93";
  *		Modified to run under 4.1c by Peter Moore 3/17/83
  */
 
+static const char *short_options = "hV";
+static struct option long_options[] = {
+  {"help", no_argument, 0, 'h'},
+  {"version", no_argument, 0, 'V'},
+  {0}
+};
+
 int
-main(int argc, char *argv[])
+main (int argc, char *argv[])
 {
 #ifndef HAVE___PROGNAME
-	extern char *__progname;
-	__progname = argv[0];
+  extern char *__progname;
+  __progname = argv[0];
 #endif
 
-	get_names(argc, argv);
-	init_display();
-	open_ctl();
-	open_sockt();
-	start_msgs();
-	if (!check_local())
-		invite_remote();
-	end_msgs();
-	set_edit_chars();
-	talk();
+  int c;
+
+  while ((c = getopt_long (argc, argv, short_options, long_options, NULL))
+	 != EOF)
+    {
+      switch (c)
+	{
+	case 'V':
+	  printf ("talk (%s %s)\n", PACKAGE_NAME, PACKAGE_VERSION);
+	  exit (0);
+
+	case 'h':
+	default:
+	  usage ();
+	  exit (0);
+	}
+    }
+
+  get_names (argc, argv);
+  init_display ();
+  open_ctl ();
+  open_sockt ();
+  start_msgs ();
+  if (!check_local ())
+    invite_remote ();
+  end_msgs ();
+  set_edit_chars ();
+  talk ();
+}
+
+
+static const char usage_str[] =
+  "Usage: talk [OPTIONS...] USER\n"
+  "\n"
+  "Options are:\n"
+  "       --help              Display usage instructions\n"
+  "       --version           Display program version\n";
+
+void
+usage (void)
+{
+  printf ("%s\n" "Send bug reports to <%s>\n", usage_str, PACKAGE_BUGREPORT);
 }
