@@ -733,14 +733,17 @@ is_unique(register char *name, register char **as, register char **ae)
 	return (1);
 }
 
-#ifdef	TERMCAP
+
 char termbuf[1024];
 
-	/*ARGSUSED*/
 int
 init_term (char *tname, int fd, int *errp)
 {
-	if (tgetent(termbuf, tname) == 1) {
+  int err = -1;
+#ifdef HAVE_LIBREADLINE
+  err = tgetent (termbuf, tname);
+#endif
+	if (err == 1) {
 		termbuf[1023] = '\0';
 		if (errp)
 			*errp = 1;
@@ -750,10 +753,6 @@ init_term (char *tname, int fd, int *errp)
 		*errp = 0;
 	return(-1);
 }
-#else
-#define	termbuf	ttytype
-extern char ttytype[];
-#endif
 
 int resettermname = 1;
 
@@ -1308,7 +1307,7 @@ slc_export()
 	}
     }
     slc_end_reply();
-    (void)slc_update();
+    slc_update();
     setconnmode(1);	/* Make sure the character values are set */
 }
 
@@ -1829,7 +1828,7 @@ process_iac:
 		     */
 		printoption("RCVD", IAC, DM);
 		SYNCHing = 1;
-		(void) ttyflush(1);
+		ttyflush(1);
 		SYNCHing = stilloob();
 		settimer(gotDM);
 		break;
@@ -2171,7 +2170,7 @@ Scheduler(int block)
 #   if defined(TN3270) && defined(unix)
     if (HaveInput) {
 	HaveInput = 0;
-	(void) signal(SIGIO, inputAvailable);
+	signal(SIGIO, inputAvailable);
     }
 #endif	/* defined(TN3270) && defined(unix) */
 
@@ -2421,7 +2420,7 @@ doflush()
     NETADD(TELOPT_TM);
     flushline = 1;
     flushout = 1;
-    (void) ttyflush(1);			/* Flush/drop output */
+    ttyflush(1);			/* Flush/drop output */
     /* do printoption AFTER flush, otherwise the output gets tossed... */
     printoption("SENT", DO, TELOPT_TM);
 }
